@@ -1,7 +1,8 @@
 /*
  *
  * Copyright (C) 2003-2008 Sebastian Trueg <trueg@k3b.org>
- *           (C) 2009      Gustavo Pichorim Boiko <gustavo.boiko@kdemail.net>
+ * Copyright (C) 2009      Gustavo Pichorim Boiko <gustavo.boiko@kdemail.net>
+ * Copyright (C) 2010      Michal Malek <michalm@jabster.pl>
  *
  * This file is part of the K3b project.
  * Copyright (C) 1998-2009 Sebastian Trueg <trueg@k3b.org>
@@ -17,21 +18,30 @@
 #include "k3baudiotrack.h"
 #include "k3baudiodoc.h"
 #include "k3baudiodatasource.h"
+#include "k3baudiotrackreader.h"
 
 #include "k3baudiodecoder.h"
 #include "k3bcore.h"
 #include "k3bcdtextvalidator.h"
 
-#include <qstring.h>
-
-#include <kdebug.h>
+#include <QtCore/QDebug>
+#include <QtCore/QString>
 
 
 
 class K3b::AudioTrack::Private
 {
 public:
-    Private() {
+    Private( AudioDoc* p = 0 )
+    :
+      parent(p),
+      copy(false),
+      preEmp(false),
+      index0Offset(150),
+      prev(0),
+      next(0),
+      firstSource(0),
+      currentlyDeleting(false) {
         cdTextValidator = new K3b::CdTextValidator();
     }
 
@@ -39,69 +49,75 @@ public:
         delete cdTextValidator;
     }
 
+    AudioDoc* parent;
+
+    /** copy protection */
+    bool copy;
+    bool preEmp;
+
+    Msf index0Offset;
+
+    Device::TrackCdText cdText;
+
+    // list
+    AudioTrack* prev;
+    AudioTrack* next;
+
+    AudioDataSource* firstSource;
+
+    bool currentlyDeleting;
+
     K3b::CdTextValidator* cdTextValidator;
 };
 
 
 K3b::AudioTrack::AudioTrack()
     : QObject(),
-      m_parent(0),
-      m_copy(false),
-      m_preEmp(false),
-      m_index0Offset(150),
-      m_prev(0),
-      m_next(0),
-      m_firstSource(0),
-      m_currentSource(0),
-      m_alreadyReadBytes(0),
-      m_currentlyDeleting(false)
+      d( new Private )
 {
-    d = new Private;
 }
 
 
 K3b::AudioTrack::AudioTrack( K3b::AudioDoc* parent )
     : QObject(),
-      m_parent(parent),
-      m_copy(false),
-      m_preEmp(false),
-      m_index0Offset(150),
-      m_prev(0),
-      m_next(0),
-      m_firstSource(0),
-      m_currentSource(0),
-      m_alreadyReadBytes(0),
-      m_currentlyDeleting(false)
+      d( new Private( parent ) )
 {
-    d = new Private;
 }
 
 
 K3b::AudioTrack::~AudioTrack()
 {
-    kDebug() << this;
+    qDebug() << this;
 
-    m_currentlyDeleting = true;
+    d->currentlyDeleting = true;
 
     // fix the list
     take();
 
-    kDebug() << "deleting sources.";
+    qDebug() << "deleting sources.";
 
     // delete all sources
-    while( m_firstSource )
-        delete m_firstSource;
+    while( d->firstSource )
+        delete d->firstSource;
 
-    kDebug() << "finished";
+    qDebug() << "finished";
 
     delete d;
 }
 
 
+K3b::AudioDoc* K3b::AudioTrack::doc() const
+{
+    return d->parent;
+}
+
+
 void K3b::AudioTrack::emitChanged()
 {
-    if( m_parent && !m_currentlyDeleting )
-        m_parent->slotTrackChanged( this );
+    emit changed();
+
+    if( d->parent && !d->currentlyDeleting )
+        d->parent->slotTrackChanged( this );
 }
 
 
@@ -113,75 +129,113 @@ void K3b::AudioTrack::setArtist( const QString& a )
 
 void K3b::AudioTrack::setPerformer( const QString& a )
 {
-    QString s( a );
-    d->cdTextValidator->fixup( s );
-    m_cdText.setPerformer(s);
-    emitChanged();
+    if( performer() != a ) {
+        QString s( a );
+        d->cdTextValidator->fixup( s );
+        d->cdText.setPerformer(s);
+        emitChanged();
+    }
 }
 
 
 void K3b::AudioTrack::setTitle( const QString& t )
 {
-    QString s( t );
-    d->cdTextValidator->fixup( s );
-    m_cdText.setTitle(s);
-    emitChanged();
+    if( title() != t ) {
+        QString s( t );
+        d->cdTextValidator->fixup( s );
+        d->cdText.setTitle(s);
+        emitChanged();
+    }
 }
 
 
 void K3b::AudioTrack::setArranger( const QString& t )
 {
-    QString s( t );
-    d->cdTextValidator->fixup( s );
-    m_cdText.setArranger(s);
-    emitChanged();
+    if( arranger() != t ) {
+        QString s( t );
+        d->cdTextValidator->fixup( s );
+        d->cdText.setArranger(s);
+        emitChanged();
+    }
 }
 
 
 void K3b::AudioTrack::setSongwriter( const QString& t )
 {
-    QString s( t );
-    d->cdTextValidator->fixup( s );
-    m_cdText.setSongwriter(s);
-    emitChanged();
+    if( songwriter() != t ) {
+        QString s( t );
+        d->cdTextValidator->fixup( s );
+        d->cdText.setSongwriter(s);
+        emitChanged();
+    }
 }
 
 
 void K3b::AudioTrack::setComposer( const QString& t )
 {
-    QString s( t );
-    d->cdTextValidator->fixup( s );
-    m_cdText.setComposer(s);
-    emitChanged();
+    if( composer() != t ) {
+        QString s( t );
+        d->cdTextValidator->fixup( s );
+        d->cdText.setComposer(s);
+        emitChanged();
+    }
 }
 
 
 void K3b::AudioTrack::setIsrc( const QString& t )
 {
-    m_cdText.setIsrc(t);
-    emitChanged();
+    if( isrc() != t ) {
+        d->cdText.setIsrc(t);
+        emitChanged();
+    }
 }
 
 
 void K3b::AudioTrack::setCdTextMessage( const QString& t )
 {
-    QString s( t );
-    d->cdTextValidator->fixup( s );
-    m_cdText.setMessage(s);
-    emitChanged();
+    if( cdTextMessage() != t ) {
+        QString s( t );
+        d->cdTextValidator->fixup( s );
+        d->cdText.setMessage(s);
+        emitChanged();
+    }
 }
 
 
 void K3b::AudioTrack::setCdText( const K3b::Device::TrackCdText& cdtext )
 {
-    m_cdText = cdtext;
+    d->cdText = cdtext;
     emitChanged();
 }
 
 
+void K3b::AudioTrack::setPreEmp( bool b )
+{
+    if( d->preEmp != b ) {
+        d->preEmp = b;
+        emitChanged();
+    }
+}
+
+
+void K3b::AudioTrack::setCopyProtection( bool b )
+{
+    if( d->copy != b ) {
+        d->copy = b;
+        emitChanged();
+    }
+}
+
+
+K3b::AudioDataSource* K3b::AudioTrack::firstSource() const
+{
+    return d->firstSource;
+}
+
+
 K3b::AudioDataSource* K3b::AudioTrack::lastSource() const
 {
-    K3b::AudioDataSource* s = m_firstSource;
+    K3b::AudioDataSource* s = d->firstSource;
     while( s && s->next() )
         s = s->next();
     return s;
@@ -191,7 +245,7 @@ K3b::AudioDataSource* K3b::AudioTrack::lastSource() const
 bool K3b::AudioTrack::inList() const
 {
     if( doc() )
-        return ( doc()->firstTrack() == this || m_prev != 0 );
+        return ( doc()->firstTrack() == this || d->prev != 0 );
     else
         return false;
 }
@@ -200,7 +254,7 @@ bool K3b::AudioTrack::inList() const
 K3b::Msf K3b::AudioTrack::length() const
 {
     K3b::Msf length;
-    K3b::AudioDataSource* source = m_firstSource;
+    K3b::AudioDataSource* source = d->firstSource;
     while( source ) {
         length += source->length();
         source = source->next();
@@ -214,11 +268,76 @@ KIO::filesize_t K3b::AudioTrack::size() const
     return length().audioBytes();
 }
 
+QString K3b::AudioTrack::artist() const
+{
+    return d->cdText.performer();
+}
+
+
+QString K3b::AudioTrack::performer() const
+{
+    return d->cdText.performer();
+}
+
+
+QString K3b::AudioTrack::title() const
+{
+    return d->cdText.title();
+}
+
+
+QString K3b::AudioTrack::arranger() const
+{
+    return d->cdText.arranger();
+}
+
+
+QString K3b::AudioTrack::songwriter() const
+{
+    return d->cdText.songwriter();
+}
+
+
+QString K3b::AudioTrack::composer() const
+{
+    return d->cdText.composer();
+}
+
+
+QString K3b::AudioTrack::isrc() const
+{
+    return d->cdText.isrc();
+}
+
+
+QString K3b::AudioTrack::cdTextMessage() const
+{
+    return d->cdText.message();
+}
+
+
+K3b::Device::TrackCdText K3b::AudioTrack::cdText() const
+{
+    return d->cdText;
+}
+
+
+bool K3b::AudioTrack::copyProtection() const
+{
+    return d->copy;
+}
+
+
+bool K3b::AudioTrack::preEmp() const
+{
+    return d->preEmp;
+}
+
 
 unsigned int K3b::AudioTrack::trackNumber() const
 {
-    if( m_prev )
-        return m_prev->trackNumber() + 1;
+    if( d->prev )
+        return d->prev->trackNumber() + 1;
     else
         return 1;
 }
@@ -229,14 +348,14 @@ K3b::Msf K3b::AudioTrack::index0() const
     // we save the index0Offset as length of the resulting pregap
     // this way the length of the track does not need to be ready
     // when creating the track.
-    return length() - m_index0Offset;
+    return length() - d->index0Offset;
 }
 
 
 K3b::Msf K3b::AudioTrack::postGap() const
 {
     if( next() )
-        return m_index0Offset;
+        return d->index0Offset;
     else
         return 0;
 }
@@ -245,35 +364,36 @@ K3b::Msf K3b::AudioTrack::postGap() const
 void K3b::AudioTrack::setIndex0( const K3b::Msf& msf )
 {
     if( msf == 0 )
-        m_index0Offset = 0;
+        d->index0Offset = 0;
     else
-        m_index0Offset = length() - msf;
+        d->index0Offset = length() - msf;
 }
 
 
 K3b::AudioTrack* K3b::AudioTrack::take()
 {
     if( inList() ) {
+        const int position = trackNumber() - 1;
         if ( doc() )
-            emit doc()->aboutToRemoveTrack(this);
+            emit doc()->trackAboutToBeRemoved( position );
 
-        if( !m_prev )
-            doc()->setFirstTrack( m_next );
-        if( !m_next )
-            doc()->setLastTrack( m_prev );
+        if( !d->prev )
+            doc()->setFirstTrack( d->next );
+        if( !d->next )
+            doc()->setLastTrack( d->prev );
 
-        if( m_prev )
-            m_prev->m_next = m_next;
-        if( m_next )
-            m_next->m_prev = m_prev;
+        if( d->prev )
+            d->prev->d->next = d->next;
+        if( d->next )
+            d->next->d->prev = d->prev;
 
-        m_prev = m_next = 0;
+        d->prev = d->next = 0;
 
         // remove from doc
         if( doc() )
-            doc()->slotTrackRemoved(this);
+            doc()->slotTrackRemoved( position );
 
-        m_parent = 0;
+        d->parent = 0;
     }
 
     return this;
@@ -282,10 +402,10 @@ K3b::AudioTrack* K3b::AudioTrack::take()
 
 void K3b::AudioTrack::moveAfter( K3b::AudioTrack* track )
 {
-    kDebug() << "(K3b::AudioTrack::moveAfter( " << track << " )";
+    qDebug() << "(K3b::AudioTrack::moveAfter( " << track << " )";
     if( !track ) {
         if( !doc() ) {
-            kDebug() << "(K3b::AudioTrack::moveAfter) no parent set";
+            qDebug() << "(K3b::AudioTrack::moveAfter) no parent set";
             return;
         }
 
@@ -293,42 +413,42 @@ void K3b::AudioTrack::moveAfter( K3b::AudioTrack* track )
         if( doc()->lastTrack() )
             moveAfter( doc()->lastTrack() );
         else {
-            emit doc()->aboutToAddTrack( 0 );
+            emit doc()->trackAboutToBeAdded( 0 );
             doc()->setFirstTrack( take() );
             doc()->setLastTrack( this );
-            emit doc()->trackAdded( this );
+            emit doc()->trackAdded( 0 );
         }
     }
     else if( track == this ) {
-        kDebug() << "(K3b::AudioTrack::moveAfter) trying to move this after this.";
+        qDebug() << "(K3b::AudioTrack::moveAfter) trying to move this after this.";
         return;
     }
     else {
         // remove this from the list
         take();
-        
-        emit track->doc()->aboutToAddTrack( track->trackNumber()-1 );
+
+        emit track->doc()->trackAboutToBeAdded( track->trackNumber()-1 );
 
         // set the new parent doc
-        m_parent = track->doc();
+        d->parent = track->doc();
 
-        K3b::AudioTrack* oldNext = track->m_next;
+        K3b::AudioTrack* oldNext = track->d->next;
 
         // set track as prev
-        track->m_next = this;
-        m_prev = track;
+        track->d->next = this;
+        d->prev = track;
 
         // set oldNext as next
         if( oldNext )
-            oldNext->m_prev = this;
-        m_next = oldNext;
+            oldNext->d->prev = this;
+        d->next = oldNext;
 
-        if( !m_prev )
+        if( !d->prev )
             doc()->setFirstTrack( this );
-        if( !m_next )
+        if( !d->next )
             doc()->setLastTrack( this );
 
-        emit doc()->trackAdded( this );
+        emit doc()->trackAdded( track->trackNumber()-1 );
     }
 
     emitChanged();
@@ -339,7 +459,7 @@ void K3b::AudioTrack::moveAhead( K3b::AudioTrack* track )
 {
     if( !track ) {
         if( !doc() ) {
-            kDebug() << "(K3b::AudioTrack::moveAfter) no parent set";
+            qDebug() << "(K3b::AudioTrack::moveAfter) no parent set";
             return;
         }
 
@@ -347,42 +467,42 @@ void K3b::AudioTrack::moveAhead( K3b::AudioTrack* track )
         if( doc()->firstTrack() )
             moveAhead( doc()->firstTrack() );
         else {
-            emit doc()->aboutToAddTrack( 0 );
+            emit doc()->trackAboutToBeAdded( 0 );
             doc()->setFirstTrack( take() );
             doc()->setLastTrack( this );
-            emit doc()->trackAdded( this );
+            emit doc()->trackAdded( 0 );
         }
     }
     else if( track == this ) {
-        kDebug() << "(K3b::AudioTrack::moveAhead) trying to move this ahead of this.";
+        qDebug() << "(K3b::AudioTrack::moveAhead) trying to move this ahead of this.";
         return;
     }
     else {
         // remove this from the list
         take();
-        
-        emit track->doc()->aboutToAddTrack( track->trackNumber()-1 );
+
+        emit track->doc()->trackAboutToBeAdded( track->trackNumber()-1 );
 
         // set the new parent doc
-        m_parent = track->doc();
+        d->parent = track->doc();
 
-        K3b::AudioTrack* oldPrev = track->m_prev;
+        K3b::AudioTrack* oldPrev = track->d->prev;
 
         // set track as next
-        m_next = track;
-        track->m_prev = this;
+        d->next = track;
+        track->d->prev = this;
 
         // set oldPrev as prev
-        m_prev = oldPrev;
+        d->prev = oldPrev;
         if( oldPrev )
-            oldPrev->m_next = this;
+            oldPrev->d->next = this;
 
-        if( !m_prev )
+        if( !d->prev )
             doc()->setFirstTrack( this );
-        if( !m_next )
+        if( !d->next )
             doc()->setLastTrack( this );
 
-        emit doc()->trackAdded( this );
+        emit doc()->trackAdded( track->trackNumber()-1 );
     }
 
     emitChanged();
@@ -391,9 +511,9 @@ void K3b::AudioTrack::moveAhead( K3b::AudioTrack* track )
 
 void K3b::AudioTrack::merge( K3b::AudioTrack* trackToMerge, K3b::AudioDataSource* sourceAfter )
 {
-    kDebug() << "(K3b::AudioTrack::merge) " << trackToMerge << " into " << this;
+    qDebug() << "(K3b::AudioTrack::merge) " << trackToMerge << " into " << this;
     if( this == trackToMerge ) {
-        kDebug() << "(K3b::AudioTrack::merge) trying to merge this with this.";
+        qDebug() << "(K3b::AudioTrack::merge) trying to merge this with this.";
         return;
     }
 
@@ -402,21 +522,21 @@ void K3b::AudioTrack::merge( K3b::AudioTrack* trackToMerge, K3b::AudioDataSource
 
     // in case we prepend all of trackToMerge's sources
     if( !sourceAfter ) {
-        kDebug() << "(K3b::AudioTrack::merge) merging " << trackToMerge->firstSource();
-        if( m_firstSource ) {
-            trackToMerge->firstSource()->moveAhead( m_firstSource );
+        qDebug() << "(K3b::AudioTrack::merge) merging " << trackToMerge->firstSource();
+        if( d->firstSource ) {
+            trackToMerge->firstSource()->moveAhead( d->firstSource );
         }
         else {
             addSource( trackToMerge->firstSource()->take() );
         }
-        sourceAfter = m_firstSource;
+        sourceAfter = d->firstSource;
     }
 
-    kDebug() << "(K3b::AudioTrack::merge) now merge the other sources.";
+    qDebug() << "(K3b::AudioTrack::merge) now merge the other sources.";
     // now merge all sources into this track
     while( trackToMerge->firstSource() ) {
         K3b::AudioDataSource* s = trackToMerge->firstSource();
-        kDebug() << "(K3b::AudioTrack::merge) merging source " << s << " from track " << s->track() << " into track "
+        qDebug() << "(K3b::AudioTrack::merge) merging source " << s << " from track " << s->track() << " into track "
                  << this << " after source " << sourceAfter << endl;
         s->moveAfter( sourceAfter );
         sourceAfter = s;
@@ -427,19 +547,27 @@ void K3b::AudioTrack::merge( K3b::AudioTrack* trackToMerge, K3b::AudioDataSource
     // now we can safely delete the track we merged
     delete trackToMerge;
 
-    kDebug() << "(K3b::AudioTrack::merge) finished";
+    qDebug() << "(K3b::AudioTrack::merge) finished";
 
     emitChanged();
 }
 
 
+K3b::AudioTrack* K3b::AudioTrack::prev() const
+{
+    return d->prev;
+}
+
+
+K3b::AudioTrack* K3b::AudioTrack::next() const
+{
+    return d->next;
+}
+
+
 void K3b::AudioTrack::setFirstSource( K3b::AudioDataSource* source )
 {
-    // reset the reading stuff since this might be a completely new source list
-    m_currentSource = 0;
-    m_alreadyReadBytes = 0;
-
-    m_firstSource = source;
+    d->firstSource = source;
     while( source ) {
         source->m_track = this;
         source = source->next();
@@ -454,7 +582,7 @@ void K3b::AudioTrack::addSource( K3b::AudioDataSource* source )
     if( !source )
         return;
 
-    K3b::AudioDataSource* s = m_firstSource;
+    K3b::AudioDataSource* s = d->firstSource;
     while( s && s->next() )
         s = s->next();
     if( s )
@@ -466,13 +594,13 @@ void K3b::AudioTrack::addSource( K3b::AudioDataSource* source )
 
 void K3b::AudioTrack::sourceChanged( K3b::AudioDataSource* )
 {
-    if( m_currentlyDeleting )
+    if( d->currentlyDeleting )
         return;
 
     // TODO: update indices
 
-    if( m_index0Offset > length() )
-        m_index0Offset = length()-1;
+    if( d->index0Offset > length() )
+        d->index0Offset = length()-1;
 
     emitChanged();
 }
@@ -480,7 +608,7 @@ void K3b::AudioTrack::sourceChanged( K3b::AudioDataSource* )
 
 int K3b::AudioTrack::numberSources() const
 {
-    K3b::AudioDataSource* source = m_firstSource;
+    K3b::AudioDataSource* source = d->firstSource;
     int i = 0;
     while( source ) {
         source = source->next();
@@ -490,59 +618,15 @@ int K3b::AudioTrack::numberSources() const
 }
 
 
-bool K3b::AudioTrack::seek( const K3b::Msf& msf )
-{
-    K3b::AudioDataSource* source = m_firstSource;
-
-    K3b::Msf pos;
-    while( source && pos + source->length() < msf ) {
-        pos += source->length();
-        source = source->next();
-    }
-
-    if( source ) {
-        m_currentSource = source;
-        m_alreadyReadBytes = msf.audioBytes();
-        return source->seek( msf - pos );
-    }
-    else
-        return false;
-}
-
-
-int K3b::AudioTrack::read( char* data, unsigned int max )
-{
-    if( !m_currentSource ) {
-        m_currentSource = m_firstSource;
-        if( m_currentSource )
-            m_currentSource->seek(0);
-        m_alreadyReadBytes = 0;
-    }
-
-    int readData = m_currentSource->read( data, max );
-    if( readData == 0 ) {
-        m_currentSource = m_currentSource->next();
-        if( m_currentSource ) {
-            m_currentSource->seek(0);
-            return read( data, max ); // read from next source
-        }
-    }
-
-    m_alreadyReadBytes += readData;
-
-    return readData;
-}
-
-
 K3b::AudioTrack* K3b::AudioTrack::copy() const
 {
     K3b::AudioTrack* track = new K3b::AudioTrack();
 
-    track->m_copy = m_copy;
-    track->m_preEmp = m_preEmp;
-    track->m_index0Offset = m_index0Offset;
-    track->m_cdText = m_cdText;
-    K3b::AudioDataSource* source = m_firstSource;
+    track->d->copy = d->copy;
+    track->d->preEmp = d->preEmp;
+    track->d->index0Offset = d->index0Offset;
+    track->d->cdText = d->cdText;
+    K3b::AudioDataSource* source = d->firstSource;
     while( source ) {
         track->addSource( source->copy() );
         source = source->next();
@@ -570,12 +654,13 @@ K3b::AudioTrack* K3b::AudioTrack::split( const K3b::Msf& pos )
             splitSource = source;
         }
         else {
-            splitSource = source->split( pos - currentPos );
+            if (source) 
+                splitSource = source->split( pos - currentPos );
         }
 
         // the new track should include all sources from splitSource and below
         K3b::AudioTrack* splitTrack = new K3b::AudioTrack();
-        splitTrack->m_cdText = m_cdText;
+        splitTrack->d->cdText = d->cdText;
         source = splitSource;
         while( source ) {
             K3b::AudioDataSource* addSource = source;
@@ -583,7 +668,7 @@ K3b::AudioTrack* K3b::AudioTrack::split( const K3b::Msf& pos )
             splitTrack->addSource( addSource );
         }
 
-        kDebug() << "(K3b::AudioTrack) moving track " << splitTrack << " after this (" << this << ") with parent " << doc();
+        qDebug() << "(K3b::AudioTrack) moving track " << splitTrack << " after this (" << this << ") with parent " << doc();
         splitTrack->moveAfter( this );
 
         return splitTrack;
@@ -593,6 +678,12 @@ K3b::AudioTrack* K3b::AudioTrack::split( const K3b::Msf& pos )
 }
 
 
+QIODevice* K3b::AudioTrack::createReader( QObject* parent )
+{
+    return new AudioTrackReader( *this, parent );
+}
+
+
 K3b::Device::Track K3b::AudioTrack::toCdTrack() const
 {
     if( !inList() )
@@ -628,13 +719,13 @@ K3b::Device::Track K3b::AudioTrack::toCdTrack() const
 
 void K3b::AudioTrack::debug()
 {
-    kDebug() << "Track " << this << endl
-             << "  Prev: " << m_prev << endl
-             << "  Next: " << m_next << endl
+    qDebug() << "Track " << this << endl
+             << "  Prev: " << d->prev << endl
+             << "  Next: " << d->next << endl
              << "  Sources:" << endl;
-    K3b::AudioDataSource* s = m_firstSource;
+    K3b::AudioDataSource* s = d->firstSource;
     while( s ) {
-        kDebug() << "  " << s << " - Prev: " << s->prev() << " Next: " << s->next();
+        qDebug() << "  " << s << " - Prev: " << s->prev() << " Next: " << s->next();
         s = s->next();
     }
 }
@@ -652,18 +743,29 @@ K3b::AudioDataSource* K3b::AudioTrack::getSource( int index ) const
 }
 
 
-void K3b::AudioTrack::emitSourceRemoved( K3b::AudioDataSource *source )
+void K3b::AudioTrack::emitSourceAboutToBeRemoved( AudioDataSource* source )
+{
+    emit sourceAboutToBeRemoved( source->sourceIndex() );
+
+    if ( doc() ) {
+        emit doc()->sourceAboutToBeRemoved( this, source->sourceIndex() );
+    }
+}
+
+
+void K3b::AudioTrack::emitSourceRemoved( K3b::AudioDataSource* source )
 {
     if ( doc() ) {
         // set the first source by hand (without using setFirstSource() )
         // just to avoid the model to read invalid firstSources
         if ( !source->prev() )
-            m_firstSource = source->next();
-
-        emit doc()->sourceRemoved( this );
+            d->firstSource = source->next();
 
+        emit doc()->sourceRemoved( this, source->sourceIndex() );
     }
 
+    emit sourceRemoved( source->sourceIndex() );
+
     // and now call the setFirstSource() to make sure the proper signals
     // are emitted
     if ( !source->prev() )
@@ -671,18 +773,12 @@ void K3b::AudioTrack::emitSourceRemoved( K3b::AudioDataSource *source )
 }
 
 
-void K3b::AudioTrack::emitAboutToRemoveSource( AudioDataSource* source )
+void K3b::AudioTrack::emitSourceAboutToBeAdded( int position )
 {
-    if ( doc() ) {
-        emit doc()->aboutToRemoveSource( this, source->sourceIndex() );
-    }
-}
+    emit sourceAboutToBeAdded( position );
 
-
-void K3b::AudioTrack::emitAboutToAddSource( int position )
-{
     if ( doc() ) {
-        emit doc()->aboutToAddSource( this, position );
+        emit doc()->sourceAboutToBeAdded( this, position );
     }
 }
 
@@ -692,8 +788,22 @@ void K3b::AudioTrack::emitSourceAdded( AudioDataSource* source )
     if ( doc() ) {
         emit doc()->sourceAdded( this, source->sourceIndex() );
         doc()->slotTrackChanged( this );
+    } else {
+        emit sourceAdded( source->sourceIndex() );
     }
 }
 
 
-#include "k3baudiotrack.moc"
+void K3b::AudioTrack::setIndex0Offset( const Msf& index0Offset )
+{
+    d->index0Offset = index0Offset;
+}
+
+
+void K3b::AudioTrack::setParent( K3b::AudioDoc* parent )
+{
+    d->parent = parent;
+}
+
+
+
