@@ -13,6 +13,7 @@
  * See the file "COPYING" for the exact licensing terms.
  */
 #include "k3bffmpegwrapper.h"
+#include "k3bplugin_i18n.h"
 
 #include <config-k3b.h>
 
@@ -35,8 +36,6 @@ extern "C" {
 
 #include <string.h>
 
-#include <klocale.h>
-
 
 #if LIBAVFORMAT_BUILD < 4629
 #define FFMPEG_CODEC(s) (&s->codec)
@@ -86,8 +85,12 @@ public:
     K3b::Msf length;
 
     // for decoding. ffmpeg requires 16-byte alignment.
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO4
+    ::AVFrame* frame;
+#else
     char outputBuffer[AVCODEC_MAX_AUDIO_FRAME_SIZE + 15];
     char* alignedOutputBuffer;
+#endif
     char* outputBufferPos;
     int outputBufferSize;
     ::AVPacket packet;
@@ -102,14 +105,29 @@ K3bFFMpegFile::K3bFFMpegFile( const QString& filename )
     d = new Private;
     d->formatContext = 0;
     d->codec = 0;
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO4
+#  if LIBAVCODEC_BUILD < AV_VERSION_INT(55,28,1)
+    d->frame = avcodec_alloc_frame();
+#  else
+    d->frame = av_frame_alloc();
+#  endif
+#else
     int offset = 0x10 - (reinterpret_cast<intptr_t>(&d->outputBuffer) & 0xf);
     d->alignedOutputBuffer = &d->outputBuffer[offset];
+#endif
 }
 
 
 K3bFFMpegFile::~K3bFFMpegFile()
 {
     close();
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO4
+#  if LIBAVCODEC_BUILD < AV_VERSION_INT(55,28,1)
+    av_free(d->frame);
+#  else
+    av_frame_free(&d->frame);
+#  endif
+#endif
     delete d;
 }
 
@@ -121,7 +139,7 @@ bool K3bFFMpegFile::open()
     // open the file
     int err = ::avformat_open_input( &d->formatContext, m_filename.toLocal8Bit(), 0, 0 );
     if( err < 0 ) {
-        kDebug() << "(K3bFFMpegFile) unable to open " << m_filename << " with error " << err;
+        qDebug() << "(K3bFFMpegFile) unable to open " << m_filename << " with error " << err;
         return false;
     }
 
@@ -130,7 +148,7 @@ bool K3bFFMpegFile::open()
 
     // we only handle files containing one audio stream
     if( d->formatContext->nb_streams != 1 ) {
-        kDebug() << "(K3bFFMpegFile) more than one stream in " << m_filename;
+        qDebug() << "(K3bFFMpegFile) more than one stream in " << m_filename;
         return false;
     }
 
@@ -138,21 +156,21 @@ bool K3bFFMpegFile::open()
     ::AVCodecContext* codecContext =  FFMPEG_CODEC(d->formatContext->streams[0]);
     if( codecContext->codec_type != AVMEDIA_TYPE_AUDIO)
     {
-        kDebug() << "(K3bFFMpegFile) not a simple audio stream: " << m_filename;
+        qDebug() << "(K3bFFMpegFile) not a simple audio stream: " << m_filename;
         return false;
     }
 
     // get the codec
     d->codec = ::avcodec_find_decoder(codecContext->codec_id);
     if( !d->codec ) {
-        kDebug() << "(K3bFFMpegFile) no codec found for " << m_filename;
+        qDebug() << "(K3bFFMpegFile) no codec found for " << m_filename;
         return false;
     }
 
     // open the codec on our context
-    kDebug() << "(K3bFFMpegFile) found codec for " << m_filename;
+    qDebug() << "(K3bFFMpegFile) found codec for " << m_filename;
     if( ::avcodec_open2( codecContext, d->codec, 0 ) < 0 ) {
-        kDebug() << "(K3bFFMpegDecoderFactory) could not open codec.";
+        qDebug() << "(K3bFFMpegDecoderFactory) could not open codec.";
         return false;
     }
 
@@ -160,7 +178,7 @@ bool K3bFFMpegFile::open()
     d->length = K3b::Msf::fromSeconds( (double)d->formatContext->duration / (double)AV_TIME_BASE );
 
     if( d->length == 0 ) {
-        kDebug() << "(K3bFFMpegDecoderFactory) invalid length.";
+        qDebug() << "(K3bFFMpegDecoderFactory) invalid length.";
         return false;
     }
 
@@ -240,12 +258,7 @@ QString K3bFFMpegFile::title() const
 {
     // FIXME: is this UTF8 or something??
     AVDictionaryEntry *ade = av_dict_get( d->formatContext->metadata, "TITLE", NULL, 0 );
-    if( ade == NULL )
-        return QString();
-    if( ade->value != '\0' )
-        return QString::fromLocal8Bit( ade->value );
-    else
-        return QString();
+    return ade && ade->value[0] != '\0' ? QString::fromLocal8Bit( ade->value ) : QString();
 }
 
 
@@ -253,12 +266,7 @@ QString K3bFFMpegFile::author() const
 {
     // FIXME: is this UTF8 or something??
     AVDictionaryEntry *ade = av_dict_get( d->formatContext->metadata, "ARTIST", NULL, 0 );
-    if( ade == NULL )
-        return QString();
-    if( ade->value != '\0' )
-        return QString::fromLocal8Bit( ade->value );
-    else
-        return QString();
+    return ade && ade->value[0] != '\0' ? QString::fromLocal8Bit( ade->value ) : QString();
 }
 
 
@@ -266,12 +274,7 @@ QString K3bFFMpegFile::comment() const
 {
     // FIXME: is this UTF8 or something??
     AVDictionaryEntry *ade = av_dict_get( d->formatContext->metadata, "COMMENT", NULL, 0 );
-    if( ade == NULL )
-        return QString();
-    if( ade->value != '\0' )
-        return QString::fromLocal8Bit( ade->value );
-    else
-        return QString();
+    return ade && ade->value[0] != '\0' ? QString::fromLocal8Bit( ade->value ) : QString();
 }
 
 
@@ -326,35 +329,60 @@ int K3bFFMpegFile::fillOutputBuffer()
             return 0;
         }
 
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO4
+        int gotFrame = 0;
+        int len = ::avcodec_decode_audio4(
+#else
         d->outputBufferPos = d->alignedOutputBuffer;
         d->outputBufferSize = AVCODEC_MAX_AUDIO_FRAME_SIZE;
-
-#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO3
+#  ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO3
         int len = ::avcodec_decode_audio3(
-#else
-#  ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO2
-        int len = ::avcodec_decode_audio2(
 #  else
+#    ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO2
+        int len = ::avcodec_decode_audio2(
+#    else
         int len = ::avcodec_decode_audio(
+#    endif
 #  endif
 #endif
 
             FFMPEG_CODEC(d->formatContext->streams[0]),
-            (short*)d->alignedOutputBuffer,
-            &d->outputBufferSize,
-#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO3
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO4
+            d->frame,
+            &gotFrame,
             &d->packet );
 #else
+            (short*)d->alignedOutputBuffer,
+            &d->outputBufferSize,
+#  ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO3
+            &d->packet );
+#  else
             d->packetData, d->packetSize );
+#  endif
 #endif
 
         if( d->packetSize <= 0 || len < 0 )
+#if LIBAVCODEC_VERSION_MAJOR >= 56
+            ::av_packet_unref( &d->packet );
+#else
             ::av_free_packet( &d->packet );
+#endif
         if( len < 0 ) {
-            kDebug() << "(K3bFFMpegFile) decoding failed for " << m_filename;
+            qDebug() << "(K3bFFMpegFile) decoding failed for " << m_filename;
             return -1;
         }
 
+#ifdef HAVE_FFMPEG_AVCODEC_DECODE_AUDIO4
+        if ( gotFrame ) {
+            d->outputBufferSize = ::av_samples_get_buffer_size(
+                NULL,
+                FFMPEG_CODEC(d->formatContext->streams[0])->channels,
+                d->frame->nb_samples,
+                FFMPEG_CODEC(d->formatContext->streams[0])->sample_fmt,
+                1 );
+            d->outputBufferPos = reinterpret_cast<char*>( d->frame->data[0] );
+        }
+#endif
         d->packetSize -= len;
         d->packetData += len;
     }
@@ -420,9 +448,9 @@ K3bFFMpegFile* K3bFFMpegWrapper::open( const QString& filename ) const
         // mp3 being one of them sadly. Most importantly: allow the libsndfile decoder to do
         // its thing.
         //
-        if( file->type() == CODEC_ID_WMAV1 ||
-            file->type() == CODEC_ID_WMAV2 ||
-            file->type() == CODEC_ID_AAC )
+        if( file->type() == AV_CODEC_ID_WMAV1 ||
+            file->type() == AV_CODEC_ID_WMAV2 ||
+            file->type() == AV_CODEC_ID_AAC )
 #endif
             return file;
     }
